
from SessionCollection import SessionCollection
from Session import Session
import os,string
import cPickle

class FileBoundCollection(SessionCollection):
    '''Maintains a list of sessions with copies in a directory so that
       session info can be restored when the server restarts.'''

    def __init__(self, filePath):
        SessionCollection.__init__(self)
        self.filePath = filePath
        self.prereadSessions()

    def prereadSessions(self):
        try:
            for name in os.listdir(self.filePath):
                filename = os.path.join(self.filePath, name)
                try:
                    file = open(filename, "rb")
                    session = cPickle.load(file)
                    file.close()
                    self.restoreSession(session, name)
                except:
                    # Can't read this file. Zap it!
                    os.unlink(filename)
        except OSError:
            raise OSError, "Directory %s does not exist. You must create it." % self.filePath

    def sessionChanged(self, session):
        '''Writes the session to a file for later restoration.'''
        filename = os.path.join(self.filePath, session.getId())
        file = open(filename, "w+b")
        cPickle.dump(session, file, 1)
        file.close()
        self.errorFlag = 0

    def sessionAdded(self, session):
        self.sessionChanged(session)

    def sessionDeleted(self, session):
        filename = os.path.join(self.filePath, session.getId())
        os.unlink(filename)

    def isErrorFlag(self):
        return self.errorFlag
