'''Keeps data for a persistent session.'''

from UserDict import UserDict
import string,time

class Store(UserDict):
    '''Implements storage object which acts like a python dictionary.
       Mostly copied from FSSession.py.
    '''
 
    def __getitem__(self,key):
        return self.data.get(key)
 
    get=__getitem__
 
    def __setitem__(self,key,value):
        self.data[key]=value
        self.changed()
 
    set=__setitem__
 
    def __delitem__(self,key):
        del self.data[key]
        self.changed()
 
    delete=__delitem__
 
    def clear(self):
        self.data.clear()
        self.changed()
 
    def copy(self):
        return self.data.copy()
 
    def update(self, other):
        if type(other) is type(self.data):
            self.data.update(other)
        else:
            for k, v in other.items():
                self.data[k] = v
        self.changed()
 
    def contents(self):
        '''Pretty print contents'''
        out = ''
        for k, v in self.items():
            out = '%s<li>%s = %s\n' % (out, str(k), str(v))
        return out

    def changed(self):
        ''' To be inherited. Placeholder for contents change event.'''
        pass



class Session(Store):

    def __init__(self, id, collection, timeout):
        UserDict.__init__(self)
        self.collection = collection
        self.id = id
        self.lastAccess = time.time()
        self.timeout = timeout

    def restoreToCollection(self, collection, id):
        self.collection = collection
        self.id = id
        
    def access(self):
        self.lastAccess = time.time()
        return self

    def isExpired(self, now):
        return self.lastAccess + self.timeout < now

    def changed(self):
        # Alert the collection that we have changed.
        self.collection.sessionChanged(self)

    def getId(self):
        return self.id

    def contentsInfo(self):
        t = int(self.timeout / 60)
        expireMin = int((self.lastAccess + self.timeout - time.time()) / 60)
        out = 'Timeout: %d minutes<br>\nMinutes until expiration: %d<br>\nContents:<br>\n<ul>\n' % (t, expireMin)
        out = out + self.contents() + '</ul>\n'
        return out        

    def __getstate__(self):
        return self.timeout, self.data

    def __setstate__(self, data):
        self.timeout, self.data = data
