

import whrandom, sha
from Session import Session
import time
import Sync

generator = whrandom.whrandom()

class SessionCollection(Sync.Synchronized):

    def __init__(self):
        self.nextCleanup = 0
        self.sessions = {}

    def getSession(self, id):
        if self.sessions.has_key(id):
            session = self.sessions[id]
            session.access()
            return session
        return None

    def restoreSession(self, session, id):
        '''To be used for restoration of old sessions.'''
        session.restoreToCollection(self, id)
        self.sessions[id] = session
        # Assume the session expiration is incorrect.
        session.access()

    def createSession(self, timeout):
        self.cleanup()
        id = ''
        while len(id) < 16 or self.sessions.has_key(id):
            # Generate a secure, unpredictable ID, but keep it
            # relatively short.
            id = sha.new(str(generator.random())).hexdigest()[0:16]
        session = Session(id, self, timeout)
        self.sessions[id] = session
        self.sessionAdded(session)
        return session

    def cleanup(self):
        # Removes expired sessions.
        now = time.time()
        if now >= self.nextCleanup:
            self.nextCleanup = now + 60
            for session in self.sessions.values():
                if session.isExpired(now):
                    del self.sessions[session.getId()]
                    self.sessionDeleted(session)

    def getSessionList(self):
        return self.sessions

    def sessionChanged(self, session):
        # To be inherited.
        pass
       

    def sessionDeleted(self, session):
        # To be inherited.
        pass

    def sessionAdded(self, session):
        # To be inherited.
        pass
