
__doc__ = """SessionManager product module."""
__version__ = '0.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import os,string
import App.FindHomes
from FileBoundCollection import FileBoundCollection
from Session import Session

# The collection is kept as a common instance and should not be
# pickled.  All SessionManager's use the same FileBoundCollection.
sessionStorageDir = os.path.join(INSTANCE_HOME, 'var', 'sessions')
collection = FileBoundCollection(sessionStorageDir)

manage_addSessionManagerForm = HTMLFile('managerAdd', globals())

def errorMessage(msg, action='./manage_main'):
    return MessageDialog(
        title='Error',
        message=msg,
        action='./manage_main'
    )

def manage_addSessionManager(self, id, title='', REQUEST=None):
    '''Add a SessionManager to a folder.'''
    if REQUEST is not None:
        timeout = 240
        if REQUEST.has_key('timeout'):
            timeout = string.atoi(REQUEST['timeout'])
        cookie_name = 'session_id'
        if REQUEST.has_key('cookie_name'):
            cookie_name = REQUEST['cookie_name']
        exported_object = 'session'
        if REQUEST.has_key('exported_object'):
            exported_object = REQUEST['exported_object']
        self._setObject(id, SessionManager(
            id, title, timeout, cookie_name, exported_object))
        return self.manage_main(self, REQUEST)

class SessionManager(
    OFS.SimpleItem.Item,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ): 
    '''SessionManager creates and retrieves sessions.'''

    meta_type = 'SessionManager'

    manage_options = (
	{'label': 'Properties',       'action': 'manage_main'},
	{'label': 'List Sessions',    'action': 'session_info'},
	{'label': 'Security',         'action': 'manage_access'},
        ) 

    __ac_permissions__ = (
	('View management screens', ('manage_tabs','manage_main')          ),
	('Change permissions',      ('manage_access',)                     ),
	('Manage properties',       ('manage_properties_change',)          ),
        ('List sessions',           ('list_sessions', 'show_session_info') )
	)

    def __init__(self, id, title, timeout, cookie_name, exported_object): 
        '''initialize a new instance of SessionManager'''
        self.id = id
        self.title = title
        self.timeout = timeout
        self.cookie_name = cookie_name
        self.exported_object = exported_object

    manage_main = HTMLFile('managerEdit', globals())
    session_info = HTMLFile('sessionInfo', globals())
    
    def manage_properties_change(self, title, REQUEST=None):
        '''Change properties of SessionManager.'''

        if REQUEST is not None:
            try:
                self.timeout = string.atoi(REQUEST['timeout'])
            except ValueError:
                return errorMessage('Timeout must be a number!')
            self.title = title

            if REQUEST.has_key('cookie_name'):
                cn = REQUEST['cookie_name']
                if len(cn) > 0:
                    self.cookie_name = cn
                
            if REQUEST.has_key('exported_object'):
                eo = REQUEST['exported_object']
                if len(eo) > 0:
                    self.exported_object = eo
                
            return MessageDialog(
                title = 'Edited',
                message = "Properties for %s changed." % self.id,
                action = './manage_main',
                )

    def list_sessions(self, REQUEST):
        '''Lists all sessions in all SessionManagers.'''
        myId = self.getSessionId(REQUEST)
        sessions = collection.getSessionList()
        rval = '<ul>\n'
        for id in sessions.keys():
            flag = ''
            if id == myId:
                flag = '*'
            rval = rval + '<li><a href="session_info?id=%s">%s</a> %s\n' % (id, id, flag)
        return rval + '</ul>\n'

    def show_session_info(self, REQUEST):
        'Shows info about one arbitrary session.'
        id = REQUEST['id']
        sessions = collection.getSessionList()
        if sessions.has_key(id):
            return sessions[id].contentsInfo()
        else:
            return "No such session exists: ", id

    def prepareSession(self, REQUEST):
        '''Puts the session object in the request as exported_object.'''
        session = self.getSession(REQUEST)
        if session is None:
            session = collection.createSession(self.timeout * 60)
            REQUEST.RESPONSE.setCookie(self.cookie_name,
                                       session.getId(), path='/')
        REQUEST[self.exported_object] = session
        return session

    def getSessionId(self, REQUEST):
        if REQUEST.has_key(self.cookie_name):
            return REQUEST[self.cookie_name]
        else:
            return None

    def getSession(self, REQUEST):
        '''Retrieves a session object if available.'''
        id = self.getSessionId(REQUEST)
        if id is not None:
            return collection.getSession(id)
        else:
            return None
